#!C:\bld\arcsi_1508458563242\_b_env\python.exe

"""
Module that contains the ARCSISolarIrradiance Class.
"""

############################################################################
#  arcsisolarirradiance.py
#
#  Copyright 2013 ARCSI.
#
#  ARCSI: 'Atmospheric and Radiometric Correction of Satellite Imagery'
#
#  ARCSI is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  ARCSI is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with ARCSI.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Purpose:  A class for calculating the irradiance for a specific
#           band of a sensor using the solar irradiance and the
#           sensor/band specific spectral response function normalised
#           to a range 0 to 1.
#
# Author: Pete Bunting
# Email: pfb@aber.ac.uk
# Date: 14/07/2013
# Version: 1.0
#
# History:
# Version 1.0 - Created.
#
############################################################################

# Import updated print function into python 2.7
from __future__ import print_function
# Import updated division operator into python 2.7
from __future__ import division
# Import the numpy library
import numpy
# Import the ARCSI exception class
from arcsilib.arcsiexception import ARCSIException
# Import the ARCSI utilities class
from arcsilib.arcsiutils import ARCSIUtils
# Import the python maths module.
import math
# Import python system library
import sys
# Import the python Argument parser
import argparse
# Import the arcsi version number
from arcsilib import ARCSI_VERSION

class ARCSISolarIrradiance (object):
    """
    A class which calculates the solar irradiance value for a given
    spectral response curve.
    """

    def getE490SolarSpectrum(self):
        spectrum = numpy.array([[119.5,1,0.00006],
                                [120.5,1,0.00056],
                                [121.5,1,0.0049],
                                [122.5,1,0.00118],
                                [123.5,1,0.00005],
                                [124.5,1,0.00003],
                                [125.5,1,0.00003],
                                [126.5,1,0.00004],
                                [127.5,1,0.00002],
                                [128.5,1,0.00002],
                                [129.5,1,0.00004],
                                [130.5,1,0.00012],
                                [131.5,1,0.00004],
                                [132.5,1,0.00004],
                                [133.5,1,0.00017],
                                [134.5,1,0.00005],
                                [135.5,1,0.00004],
                                [136.5,1,0.00003],
                                [137.5,1,0.00003],
                                [138.5,1,0.00004],
                                [139.5,1,0.00008],
                                [140.5,1,0.00006],
                                [141.5,1,0.00004],
                                [142.5,1,0.00005],
                                [143.5,1,0.00005],
                                [144.5,1,0.00005],
                                [145.5,1,0.00006],
                                [146.5,1,0.00007],
                                [147.5,1,0.00008],
                                [148.5,1,0.00008],
                                [149.5,1,0.00008],
                                [150.5,1,0.00009],
                                [151.5,1,0.00009],
                                [152.5,1,0.00012],
                                [153.5,1,0.00013],
                                [154.5,1,0.00021],
                                [155.5,1,0.00021],
                                [156.5,1,0.00018],
                                [157.5,1,0.00017],
                                [158.5,1,0.00017],
                                [159.5,1,0.00018],
                                [160.5,1,0.00019],
                                [161.5,1,0.00022],
                                [162.5,1,0.00025],
                                [163.5,1,0.00028],
                                [164.5,1,0.0003],
                                [165.5,1,0.00043],
                                [166.5,1,0.0004],
                                [167.5,1,0.00039],
                                [168.5,1,0.00046],
                                [169.5,1,0.00059],
                                [170.5,1,0.00066],
                                [171.5,1,0.00069],
                                [172.5,1,0.00073],
                                [173.5,1,0.00076],
                                [174.5,1,0.00091],
                                [175.5,1,0.00108],
                                [176.5,1,0.00122],
                                [177.5,1,0.0014],
                                [178.5,1,0.00154],
                                [179.5,1,0.00158],
                                [180.5,1,0.00183],
                                [181.5,1,0.00223],
                                [182.5,1,0.00224],
                                [183.5,1,0.00224],
                                [184.5,1,0.00207],
                                [185.5,1,0.00231],
                                [186.5,1,0.0027],
                                [187.5,1,0.00301],
                                [188.5,1,0.00329],
                                [189.5,1,0.00357],
                                [190.5,1,0.00376],
                                [191.5,1,0.00417],
                                [192.5,1,0.00411],
                                [193.5,1,0.00381],
                                [194.5,1,0.00521],
                                [195.5,1,0.00543],
                                [196.5,1,0.00601],
                                [197.5,1,0.00619],
                                [198.5,1,0.00619],
                                [199.5,1,0.00666],
                                [200.5,1,0.00733],
                                [201.5,1,0.00802],
                                [202.5,1,0.00826],
                                [203.5,1,0.00922],
                                [204.5,1,0.01025],
                                [205.5,1,0.01054],
                                [206.5,1,0.01108],
                                [207.5,1,0.01265],
                                [208.5,1,0.01505],
                                [209.5,1,0.02138],
                                [210.5,1,0.02792],
                                [211.5,1,0.03354],
                                [212.5,1,0.0313],
                                [213.5,1,0.03315],
                                [214.5,1,0.04003],
                                [215.5,1,0.03615],
                                [216.5,1,0.03227],
                                [217.5,1,0.03529],
                                [218.5,1,0.04437],
                                [219.5,1,0.04692],
                                [220.5,1,0.04733],
                                [221.5,1,0.03958],
                                [222.5,1,0.04965],
                                [223.5,1,0.06301],
                                [224.5,1,0.05897],
                                [225.5,1,0.05229],
                                [226.5,1,0.0394],
                                [227.5,1,0.03992],
                                [228.5,1,0.05195],
                                [229.5,1,0.04771],
                                [230.5,1,0.05212],
                                [231.5,1,0.05097],
                                [232.5,1,0.05326],
                                [233.5,1,0.04474],
                                [234.5,1,0.03897],
                                [235.5,1,0.05142],
                                [236.5,1,0.04859],
                                [237.5,1,0.04844],
                                [238.5,1,0.04196],
                                [239.5,1,0.04412],
                                [240.5,1,0.03956],
                                [241.5,1,0.05148],
                                [242.5,1,0.0706],
                                [243.5,1,0.06653],
                                [244.5,1,0.06097],
                                [245.5,1,0.04939],
                                [246.5,1,0.0504],
                                [247.5,1,0.0555],
                                [248.5,1,0.04565],
                                [249.5,1,0.05638],
                                [250.5,1,0.0601],
                                [251.5,1,0.04601],
                                [252.5,1,0.04155],
                                [253.5,1,0.05155],
                                [254.5,1,0.05957],
                                [255.5,1,0.0793],
                                [256.5,1,0.1018],
                                [257.5,1,0.1254],
                                [258.5,1,0.1251],
                                [259.5,1,0.104],
                                [260.5,1,0.08551],
                                [261.5,1,0.0898],
                                [262.5,1,0.1036],
                                [263.5,1,0.1658],
                                [264.5,1,0.2497],
                                [265.5,1,0.2527],
                                [266.5,1,0.2494],
                                [267.5,1,0.2508],
                                [268.5,1,0.2438],
                                [269.5,1,0.2389],
                                [270.5,1,0.2673],
                                [271.5,1,0.2244],
                                [272.5,1,0.1974],
                                [273.5,1,0.1965],
                                [274.5,1,0.1326],
                                [275.5,1,0.1751],
                                [276.5,1,0.2428],
                                [277.5,1,0.2338],
                                [278.5,1,0.1593],
                                [279.5,1,0.08555],
                                [280.5,1,0.09463],
                                [281.5,1,0.2083],
                                [282.5,1,0.2941],
                                [283.5,1,0.3135],
                                [284.5,1,0.2353],
                                [285.5,1,0.1631],
                                [286.5,1,0.3227],
                                [287.5,1,0.3363],
                                [288.5,1,0.3222],
                                [289.5,1,0.4727],
                                [290.5,1,0.6013],
                                [291.5,1,0.5808],
                                [292.5,1,0.5219],
                                [293.5,1,0.5355],
                                [294.5,1,0.5088],
                                [295.5,1,0.5532],
                                [296.5,1,0.5096],
                                [297.5,1,0.5073],
                                [298.5,1,0.4655],
                                [299.5,1,0.484],
                                [300.5,1,0.42],
                                [301.5,1,0.4555],
                                [302.5,1,0.489],
                                [303.5,1,0.6206],
                                [304.5,1,0.6025],
                                [305.5,1,0.5948],
                                [306.5,1,0.5557],
                                [307.5,1,0.615],
                                [308.5,1,0.6114],
                                [309.5,1,0.4965],
                                [310.5,1,0.6224],
                                [311.5,1,0.7292],
                                [312.5,1,0.6559],
                                [313.5,1,0.6999],
                                [314.5,1,0.6629],
                                [315.5,1,0.633],
                                [316.5,1,0.6332],
                                [317.5,1,0.7739],
                                [318.5,1,0.6649],
                                [319.5,1,0.7105],
                                [320.5,1,0.8051],
                                [321.5,1,0.6995],
                                [322.5,1,0.6886],
                                [323.5,1,0.6613],
                                [324.5,1,0.7608],
                                [325.5,1,0.8758],
                                [326.5,1,0.9795],
                                [327.5,1,0.9527],
                                [328.5,1,0.9176],
                                [329.5,1,1.061],
                                [330.5,1,1.016],
                                [331.5,1,0.9657],
                                [332.5,1,0.9549],
                                [333.5,1,0.9216],
                                [334.5,1,0.9589],
                                [335.5,1,0.9434],
                                [336.5,1,0.8095],
                                [337.5,1,0.8418],
                                [338.5,1,0.9215],
                                [339.5,1,0.9581],
                                [340.5,1,1.007],
                                [341.5,1,0.9238],
                                [342.5,1,0.993],
                                [343.5,1,0.9506],
                                [344.5,1,0.7957],
                                [345.5,1,0.9392],
                                [346.5,1,0.9264],
                                [347.5,1,0.9017],
                                [348.5,1,0.8972],
                                [349.5,1,0.8898],
                                [350.5,1,1.05],
                                [351.5,1,0.9795],
                                [352.5,1,0.9079],
                                [353.5,1,1.033],
                                [354.5,1,1.111],
                                [355.5,1,1.045],
                                [356.5,1,0.9123],
                                [357.5,1,0.796],
                                [358.5,1,0.6936],
                                [359.5,1,0.9911],
                                [360.5,1,0.9708],
                                [361.5,1,0.8781],
                                [362.5,1,0.9978],
                                [363.5,1,0.9969],
                                [364.5,1,1.013],
                                [365.5,1,1.152],
                                [366.5,1,1.233],
                                [367.5,1,1.18],
                                [368.5,1,1.101],
                                [369.5,1,1.226],
                                [370.5,1,1.139],
                                [371.5,1,1.175],
                                [372.5,1,1.054],
                                [373.5,1,0.9202],
                                [374.5,1,0.9004],
                                [375.5,1,1.062],
                                [376.5,1,1.085],
                                [377.5,1,1.282],
                                [378.5,1,1.327],
                                [379.5,1,1.066],
                                [380.5,1,1.202],
                                [381.5,1,1.082],
                                [382.5,1,0.7913],
                                [383.5,1,0.6841],
                                [384.5,1,0.9597],
                                [385.5,1,1.008],
                                [386.5,1,1.007],
                                [387.5,1,1.004],
                                [388.5,1,0.9843],
                                [389.5,1,1.174],
                                [390.5,1,1.247],
                                [391.5,1,1.342],
                                [392.5,1,1.019],
                                [393.5,1,0.5823],
                                [394.5,1,1.026],
                                [395.5,1,1.314],
                                [396.5,1,0.8545],
                                [397.5,1,0.9288],
                                [398.5,1,1.522],
                                [399.5,1,1.663],
                                [400.5,1,1.682],
                                [401.5,1,1.746],
                                [402.5,1,1.759],
                                [403.5,1,1.684],
                                [404.5,1,1.674],
                                [405.5,1,1.667],
                                [406.5,1,1.589],
                                [407.5,1,1.628],
                                [408.5,1,1.735],
                                [409.5,1,1.715],
                                [410.5,1,1.532],
                                [411.5,1,1.817],
                                [412.5,1,1.789],
                                [413.5,1,1.756],
                                [414.5,1,1.737],
                                [415.5,1,1.734],
                                [416.5,1,1.842],
                                [417.5,1,1.665],
                                [418.5,1,1.684],
                                [419.5,1,1.701],
                                [420.5,1,1.757],
                                [421.5,1,1.797],
                                [422.5,1,1.582],
                                [423.5,1,1.711],
                                [424.5,1,1.767],
                                [425.5,1,1.695],
                                [426.5,1,1.698],
                                [427.5,1,1.569],
                                [428.5,1,1.587],
                                [429.5,1,1.475],
                                [430.5,1,1.135],
                                [431.5,1,1.686],
                                [432.5,1,1.646],
                                [433.5,1,1.731],
                                [434.5,1,1.67],
                                [435.5,1,1.723],
                                [436.5,1,1.929],
                                [437.5,1,1.806],
                                [438.5,1,1.567],
                                [439.5,1,1.825],
                                [440.5,1,1.713],
                                [441.5,1,1.931],
                                [442.5,1,1.98],
                                [443.5,1,1.909],
                                [444.5,1,1.973],
                                [445.5,1,1.821],
                                [446.5,1,1.891],
                                [447.5,1,2.077],
                                [448.5,1,1.973],
                                [449.5,1,2.027],
                                [450.5,1,2.144],
                                [451.5,1,2.109],
                                [452.5,1,1.941],
                                [453.5,1,1.97],
                                [454.5,1,1.979],
                                [455.5,1,2.034],
                                [456.5,1,2.077],
                                [457.5,1,2.1],
                                [458.5,1,1.971],
                                [459.5,1,2.009],
                                [460.5,1,2.04],
                                [461.5,1,2.055],
                                [462.5,1,2.104],
                                [463.5,1,2.04],
                                [464.5,1,1.976],
                                [465.5,1,2.042],
                                [466.5,1,1.921],
                                [467.5,1,2.015],
                                [468.5,1,1.994],
                                [469.5,1,1.99],
                                [470.5,1,1.877],
                                [471.5,1,2.018],
                                [472.5,1,2.041],
                                [473.5,1,1.991],
                                [474.5,1,2.051],
                                [475.5,1,2.016],
                                [476.5,1,1.956],
                                [477.5,1,2.075],
                                [478.5,1,2.009],
                                [479.5,1,2.076],
                                [480.5,1,2.035],
                                [481.5,1,2.09],
                                [482.5,1,2.023],
                                [483.5,1,2.019],
                                [484.5,1,1.969],
                                [485.5,1,1.83],
                                [486.5,1,1.625],
                                [487.5,1,1.83],
                                [488.5,1,1.914],
                                [489.5,1,1.96],
                                [490.5,1,2.007],
                                [491.5,1,1.896],
                                [492.5,1,1.896],
                                [493.5,1,1.888],
                                [494.5,1,2.058],
                                [495.5,1,1.926],
                                [496.5,1,2.017],
                                [497.5,1,2.018],
                                [498.5,1,1.866],
                                [499.5,1,1.97],
                                [500.5,1,1.857],
                                [501.5,1,1.812],
                                [502.5,1,1.894],
                                [503.5,1,1.934],
                                [504.5,1,1.869],
                                [505.5,1,1.993],
                                [506.5,1,1.961],
                                [507.5,1,1.906],
                                [508.5,1,1.919],
                                [509.5,1,1.916],
                                [510.5,1,1.947],
                                [511.5,1,1.997],
                                [512.5,1,1.867],
                                [513.5,1,1.861],
                                [514.5,1,1.874],
                                [515.5,1,1.9],
                                [516.5,1,1.669],
                                [517.5,1,1.726],
                                [518.5,1,1.654],
                                [519.5,1,1.828],
                                [520.5,1,1.831],
                                [521.5,1,1.906],
                                [522.5,1,1.823],
                                [523.5,1,1.894],
                                [524.5,1,1.958],
                                [525.5,1,1.93],
                                [526.5,1,1.674],
                                [527.5,1,1.828],
                                [528.5,1,1.897],
                                [529.5,1,1.918],
                                [530.5,1,1.952],
                                [531.5,1,1.963],
                                [532.5,1,1.77],
                                [533.5,1,1.923],
                                [534.5,1,1.858],
                                [535.5,1,1.99],
                                [536.5,1,1.871],
                                [537.5,1,1.882],
                                [538.5,1,1.904],
                                [539.5,1,1.832],
                                [540.5,1,1.769],
                                [541.5,1,1.881],
                                [542.5,1,1.825],
                                [543.5,1,1.879],
                                [544.5,1,1.879],
                                [545.5,1,1.901],
                                [546.5,1,1.879],
                                [547.5,1,1.833],
                                [548.5,1,1.863],
                                [549.5,1,1.895],
                                [550.5,1,1.862],
                                [551.5,1,1.871],
                                [552.5,1,1.846],
                                [553.5,1,1.882],
                                [554.5,1,1.898],
                                [555.5,1,1.897],
                                [556.5,1,1.821],
                                [557.5,1,1.846],
                                [558.5,1,1.787],
                                [559.5,1,1.808],
                                [560.5,1,1.843],
                                [561.5,1,1.824],
                                [562.5,1,1.85],
                                [563.5,1,1.861],
                                [564.5,1,1.854],
                                [565.5,1,1.798],
                                [566.5,1,1.829],
                                [567.5,1,1.887],
                                [568.5,1,1.81],
                                [569.5,1,1.86],
                                [570.5,1,1.769],
                                [571.5,1,1.823],
                                [572.5,1,1.892],
                                [573.5,1,1.876],
                                [574.5,1,1.867],
                                [575.5,1,1.83],
                                [576.5,1,1.846],
                                [577.5,1,1.857],
                                [578.5,1,1.783],
                                [579.5,1,1.828],
                                [580.5,1,1.838],
                                [581.5,1,1.853],
                                [582.5,1,1.873],
                                [583.5,1,1.857],
                                [584.5,1,1.86],
                                [585.5,1,1.783],
                                [586.5,1,1.83],
                                [587.5,1,1.848],
                                [588.5,1,1.75],
                                [589.5,1,1.612],
                                [590.5,1,1.813],
                                [591.5,1,1.787],
                                [592.5,1,1.808],
                                [593.5,1,1.796],
                                [594.5,1,1.773],
                                [595.5,1,1.782],
                                [596.5,1,1.805],
                                [597.5,1,1.78],
                                [598.5,1,1.757],
                                [599.5,1,1.774],
                                [600.5,1,1.746],
                                [601.5,1,1.751],
                                [602.5,1,1.719],
                                [603.5,1,1.787],
                                [604.5,1,1.776],
                                [605.5,1,1.763],
                                [606.5,1,1.759],
                                [607.5,1,1.757],
                                [608.5,1,1.743],
                                [609.5,1,1.744],
                                [610.5,1,1.703],
                                [611.5,1,1.746],
                                [612.5,1,1.705],
                                [613.5,1,1.683],
                                [614.5,1,1.713],
                                [615.5,1,1.713],
                                [616.5,1,1.609],
                                [617.5,1,1.707],
                                [618.5,1,1.724],
                                [619.5,1,1.707],
                                [620.5,1,1.734],
                                [621.5,1,1.69],
                                [622.5,1,1.713],
                                [623.5,1,1.666],
                                [624.5,1,1.656],
                                [625.5,1,1.632],
                                [626.5,1,1.697],
                                [627.5,1,1.697],
                                [628.5,1,1.697],
                                [629.5,1.5,1.677],
                                [631,2,1.639],
                                [633,2,1.651],
                                [635,2,1.656],
                                [637,2,1.654],
                                [639,2,1.651],
                                [641,2,1.614],
                                [643,2,1.621],
                                [645,2,1.627],
                                [647,2,1.603],
                                [649,2,1.558],
                                [651,2,1.606],
                                [653,2,1.599],
                                [655,2,1.532],
                                [657,2,1.384],
                                [659,2,1.549],
                                [661,2,1.571],
                                [663,2,1.555],
                                [665,2,1.56],
                                [667,2,1.535],
                                [669,2,1.546],
                                [671,2,1.516],
                                [673,2,1.521],
                                [675,2,1.51],
                                [677,2,1.508],
                                [679,2,1.498],
                                [681,2,1.492],
                                [683,2,1.479],
                                [685,2,1.455],
                                [687,2,1.467],
                                [689,2,1.461],
                                [691,2,1.448],
                                [693,2,1.448],
                                [695,2,1.436],
                                [697,2,1.416],
                                [699,2,1.425],
                                [701,2,1.386],
                                [703,2,1.388],
                                [705,2,1.415],
                                [707,2,1.4],
                                [709,2,1.384],
                                [711,2,1.385],
                                [713,2,1.373],
                                [715,2,1.366],
                                [717,2,1.354],
                                [719,2,1.328],
                                [721,2,1.331],
                                [723,2,1.348],
                                [725,2,1.35],
                                [727,2,1.346],
                                [729,2,1.319],
                                [731,2,1.326],
                                [733,2,1.318],
                                [735,2,1.309],
                                [737,2,1.307],
                                [739,2,1.278],
                                [741,2,1.258],
                                [743,2,1.286],
                                [745,2,1.279],
                                [747,2,1.283],
                                [749,2,1.27],
                                [751,2,1.262],
                                [753,2,1.259],
                                [755,2,1.255],
                                [757,2,1.248],
                                [759,2,1.24],
                                [761,2,1.237],
                                [763,2,1.241],
                                [765,2,1.221],
                                [767,2,1.185],
                                [769,2,1.203],
                                [771,2,1.204],
                                [773,2,1.208],
                                [775,2,1.188],
                                [777,2,1.196],
                                [779,2,1.187],
                                [781,2,1.187],
                                [783,2,1.176],
                                [785,2,1.18],
                                [787,2,1.177],
                                [789,2,1.174],
                                [791,2,1.158],
                                [793,2,1.143],
                                [795,2,1.134],
                                [797,2,1.152],
                                [799,2,1.135],
                                [801,2,1.142],
                                [803,2,1.129],
                                [805,2,1.115],
                                [807,2,1.12],
                                [809,2,1.095],
                                [811,2,1.114],
                                [813,2,1.115],
                                [815,2,1.107],
                                [817,2,1.104],
                                [819,2,1.063],
                                [821,2,1.08],
                                [823,2,1.073],
                                [825,1,1.075],
                                [826,2,1.08],
                                [828,2,1.081],
                                [830,2,1.063],
                                [832,2,1.051],
                                [834,2,1.041],
                                [836,2,1.052],
                                [838,2,1.044],
                                [840,2,1.04],
                                [842,2,1.036],
                                [844,2,1.024],
                                [846,2,1.028],
                                [848,2,1.023],
                                [850,2,0.966],
                                [852,2,0.9961],
                                [854,2,0.878],
                                [856,2,0.9755],
                                [858,2,1.005],
                                [860,2,0.9969],
                                [862,2,0.9949],
                                [864,2,0.9993],
                                [866,2,0.8862],
                                [868,2,0.9395],
                                [870,2,0.9747],
                                [872,2,0.9833],
                                [874,2,0.9713],
                                [876,2,0.964],
                                [878,2,0.9749],
                                [880,2,0.9554],
                                [882,2,0.9511],
                                [884,2,0.9579],
                                [886,2,0.9383],
                                [888,2,0.9443],
                                [890,2,0.953],
                                [892,2,0.9394],
                                [894,2,0.9332],
                                [896,2,0.9387],
                                [898,2,0.9339],
                                [900,2,0.9158],
                                [902,2,0.8916],
                                [904,2,0.9285],
                                [906,2,0.9176],
                                [908,2,0.9025],
                                [910,2,0.8916],
                                [912,2,0.8967],
                                [914,2,0.9071],
                                [916,2,0.9004],
                                [918,2,0.8951],
                                [920,2,0.8908],
                                [922,2,0.863],
                                [924,2,0.8585],
                                [926,2,0.8612],
                                [928,2,0.8769],
                                [930,2,0.8677],
                                [932,2,0.8651],
                                [934,2,0.8641],
                                [936,2,0.8547],
                                [938,2,0.858],
                                [940,2,0.8438],
                                [942,2,0.825],
                                [944,2,0.8324],
                                [946,2,0.8375],
                                [948,2,0.8407],
                                [950,2,0.8369],
                                [952,2,0.8317],
                                [954,2,0.808],
                                [956,2,0.8082],
                                [958,2,0.8188],
                                [960,2,0.8151],
                                [962,2,0.8089],
                                [964,2,0.8013],
                                [966,2,0.7947],
                                [968,2,0.7969],
                                [970,2,0.7959],
                                [972,2,0.7936],
                                [974,2,0.7815],
                                [976,2,0.7825],
                                [978,2,0.7779],
                                [980,2,0.7746],
                                [982,2,0.7764],
                                [984,2,0.7698],
                                [986,2,0.7661],
                                [988,2,0.7615],
                                [990,2,0.7541],
                                [992,2,0.7567],
                                [994,2,0.7556],
                                [996,2,0.7525],
                                [998,2,0.751],
                                [1000,2,0.7479],
                                [1002,2,0.7469],
                                [1004,2,0.7261],
                                [1006,2,0.7136],
                                [1008,2,0.7335],
                                [1010,2,0.7313],
                                [1012,2,0.7262],
                                [1014,2,0.721],
                                [1016,2,0.7139],
                                [1018,2,0.7107],
                                [1020,2,0.7041],
                                [1022,2,0.7021],
                                [1024,2,0.7054],
                                [1026,2,0.7027],
                                [1028,2,0.6989],
                                [1030,2,0.6937],
                                [1032,2,0.6905],
                                [1034,2,0.6817],
                                [1036,2,0.684],
                                [1038,2,0.6772],
                                [1040,2,0.6761],
                                [1042,2,0.6746],
                                [1044,2,0.6714],
                                [1046,2,0.66],
                                [1048,2,0.6644],
                                [1050,2,0.6622],
                                [1052,2,0.6586],
                                [1054,2,0.6549],
                                [1056,2,0.6557],
                                [1058,2,0.6451],
                                [1060,2,0.6415],
                                [1062,2,0.6438],
                                [1064,2,0.6459],
                                [1066,2,0.6395],
                                [1068,2,0.6317],
                                [1070,2,0.6241],
                                [1072,2,0.6326],
                                [1074,2,0.6276],
                                [1076,2,0.628],
                                [1078,2,0.6272],
                                [1080,2,0.6247],
                                [1082,2,0.6099],
                                [1084,2,0.618],
                                [1086,2,0.6208],
                                [1088,2,0.6103],
                                [1090,2,0.6199],
                                [1092,2,0.6159],
                                [1094,2,0.5849],
                                [1096,2,0.5983],
                                [1098,2,0.5961],
                                [1100,2,0.6042],
                                [1102,2,0.5932],
                                [1104,2,0.5974],
                                [1106,2,0.5945],
                                [1108,2,0.5916],
                                [1110,2,0.5906],
                                [1112,2,0.5843],
                                [1114,2,0.5844],
                                [1116,2,0.5831],
                                [1118,2,0.5815],
                                [1120,2,0.5741],
                                [1122,2,0.5796],
                                [1124,2,0.5769],
                                [1126,2,0.5655],
                                [1128,2,0.57],
                                [1130,2,0.5653],
                                [1132,2,0.5678],
                                [1134,2,0.5638],
                                [1136,2,0.5658],
                                [1138,2,0.5569],
                                [1140,2,0.553],
                                [1142,2,0.5531],
                                [1144,2,0.5514],
                                [1146,2,0.5548],
                                [1148,2,0.5525],
                                [1150,2,0.5489],
                                [1152,2,0.5458],
                                [1154,2,0.5479],
                                [1156,2,0.5455],
                                [1158,2,0.5435],
                                [1160,2,0.532],
                                [1162,2,0.5325],
                                [1164,2,0.5332],
                                [1166,2,0.5303],
                                [1168,2,0.5312],
                                [1170,2,0.5276],
                                [1172,2,0.5315],
                                [1174,2,0.5273],
                                [1176,2,0.5184],
                                [1178,2,0.519],
                                [1180,2,0.5239],
                                [1182,2,0.5159],
                                [1184,2,0.5103],
                                [1186,2,0.5187],
                                [1188,2,0.5075],
                                [1190,2,0.5085],
                                [1192,2,0.5161],
                                [1194,2,0.5145],
                                [1196,2,0.5084],
                                [1198,2,0.4943],
                                [1200,2,0.5003],
                                [1202,2,0.5068],
                                [1204,2,0.4948],
                                [1206,2,0.5039],
                                [1208,2,0.489],
                                [1210,2,0.4882],
                                [1212,2,0.4933],
                                [1214,2,0.4942],
                                [1216,2,0.493],
                                [1218,2,0.4897],
                                [1220,2,0.4875],
                                [1222,2,0.4854],
                                [1224,2,0.4846],
                                [1226,2,0.4817],
                                [1228,2,0.4771],
                                [1230,2,0.4792],
                                [1232,2,0.475],
                                [1234,2,0.4729],
                                [1236,2,0.4719],
                                [1238,2,0.4703],
                                [1240,2,0.4653],
                                [1242,2,0.4642],
                                [1244,2,0.4619],
                                [1246,2,0.4635],
                                [1248,2,0.4633],
                                [1250,2,0.4624],
                                [1252,2,0.4571],
                                [1254,2,0.4574],
                                [1256,2,0.4551],
                                [1258,2,0.4533],
                                [1260,2,0.453],
                                [1262,2,0.4497],
                                [1264,2,0.4478],
                                [1266,2,0.4467],
                                [1268,2,0.4417],
                                [1270,2,0.4453],
                                [1272,2,0.4452],
                                [1274,2,0.4431],
                                [1276,2,0.4451],
                                [1278,2,0.444],
                                [1280,2,0.4356],
                                [1282,2,0.4014],
                                [1284,2,0.4259],
                                [1286,2,0.4328],
                                [1288,2,0.4314],
                                [1290,2,0.4255],
                                [1292,2,0.4254],
                                [1294,2,0.4223],
                                [1296,2,0.4224],
                                [1298,2,0.4184],
                                [1300,2,0.4186],
                                [1302,2,0.4139],
                                [1304,2,0.4111],
                                [1306,2,0.4136],
                                [1308,2,0.4123],
                                [1310,2,0.4106],
                                [1312,2,0.4033],
                                [1314,2,0.4022],
                                [1316,2,0.3979],
                                [1318,2,0.4017],
                                [1320,2,0.4016],
                                [1322,2,0.3986],
                                [1324,2,0.3981],
                                [1326,2,0.3949],
                                [1328,2,0.3908],
                                [1330,2,0.3878],
                                [1332,2,0.3863],
                                [1334,2,0.3892],
                                [1336,2,0.3866],
                                [1338,2,0.3832],
                                [1340,2,0.379],
                                [1342,2,0.3805],
                                [1344,2,0.3798],
                                [1346,2,0.3772],
                                [1348,2,0.3766],
                                [1350,2,0.3724],
                                [1352,2,0.3742],
                                [1354,2,0.3722],
                                [1356,2,0.3675],
                                [1358,2,0.3688],
                                [1360,2,0.3673],
                                [1362,2,0.3677],
                                [1364,2,0.3657],
                                [1366,2,0.3657],
                                [1368,2,0.3628],
                                [1370,2,0.3599],
                                [1372,2,0.3621],
                                [1374,2,0.3611],
                                [1376,2,0.3561],
                                [1378,2,0.358],
                                [1380,2,0.3579],
                                [1382,2,0.3545],
                                [1384,2,0.3547],
                                [1386,2,0.3532],
                                [1388,2,0.353],
                                [1390,2,0.3506],
                                [1392,2,0.3513],
                                [1394,2,0.3488],
                                [1396,2,0.3487],
                                [1398,2,0.3492],
                                [1400,2,0.3427],
                                [1402,2,0.3439],
                                [1404,2,0.3428],
                                [1406,2,0.3431],
                                [1408,2,0.3427],
                                [1410,2,0.3418],
                                [1412,2,0.3348],
                                [1414,2,0.3377],
                                [1416,2,0.3385],
                                [1418,2,0.3386],
                                [1420,2,0.3357],
                                [1422,2,0.3315],
                                [1424,2,0.3311],
                                [1426,2,0.3281],
                                [1428,2,0.3285],
                                [1430,2,0.3257],
                                [1432,2,0.33],
                                [1434,2,0.3284],
                                [1436,2,0.3285],
                                [1438,2,0.3283],
                                [1440,2,0.3188],
                                [1442,2,0.3186],
                                [1444,2,0.3197],
                                [1446,2,0.3216],
                                [1448,2,0.3216],
                                [1450,2,0.3187],
                                [1452,2,0.3154],
                                [1454,2,0.3143],
                                [1456,2,0.3131],
                                [1458,2,0.3167],
                                [1460,2,0.3156],
                                [1462,2,0.3121],
                                [1464,2,0.3105],
                                [1466,2,0.3108],
                                [1468,2,0.3114],
                                [1470,2,0.3102],
                                [1472,2,0.3073],
                                [1474,2,0.3034],
                                [1476,2,0.3048],
                                [1478,2,0.3044],
                                [1480,2,0.3068],
                                [1482,2,0.3044],
                                [1484,2,0.3039],
                                [1486,2,0.3033],
                                [1488,2,0.2855],
                                [1490,2,0.3015],
                                [1492,2,0.3018],
                                [1494,2,0.3033],
                                [1496,2,0.2972],
                                [1498,2,0.2994],
                                [1500,2,0.3011],
                                [1502,2,0.2924],
                                [1504,2,0.2799],
                                [1506,2,0.2848],
                                [1508,2,0.2919],
                                [1510,2,0.2947],
                                [1512,2,0.2913],
                                [1514,2,0.2883],
                                [1516,2,0.2882],
                                [1518,2,0.2884],
                                [1520,2,0.2866],
                                [1522,2,0.2824],
                                [1524,2,0.2835],
                                [1526,2,0.2846],
                                [1528,2,0.2846],
                                [1530,2,0.2765],
                                [1532,2,0.2823],
                                [1534,2,0.2784],
                                [1536,2,0.2806],
                                [1538,2,0.2773],
                                [1540,2,0.273],
                                [1542,2,0.2753],
                                [1544,2,0.2778],
                                [1546,2,0.2772],
                                [1548,2,0.2711],
                                [1550,2,0.2713],
                                [1552,2,0.2731],
                                [1554,2,0.2676],
                                [1556,2,0.2671],
                                [1558,2,0.2689],
                                [1560,2,0.2683],
                                [1562,2,0.2697],
                                [1564,2,0.2669],
                                [1566,2,0.2654],
                                [1568,2,0.2633],
                                [1570,2,0.2645],
                                [1572,2,0.2673],
                                [1574,2,0.261],
                                [1576,2,0.2536],
                                [1578,2,0.2547],
                                [1580,2,0.265],
                                [1582,2,0.259],
                                [1584,2,0.2591],
                                [1586,2,0.2599],
                                [1588,2,0.249],
                                [1590,2,0.2405],
                                [1592,2,0.2526],
                                [1594,2,0.2583],
                                [1596,2,0.2506],
                                [1598,2,0.2545],
                                [1600,2,0.2512],
                                [1602,2,0.2489],
                                [1604,2,0.2497],
                                [1606,2,0.2477],
                                [1608,2,0.2491],
                                [1610,2,0.24],
                                [1612,2,0.243],
                                [1614,2,0.2449],
                                [1616,2,0.2374],
                                [1618,2,0.2423],
                                [1620,2,0.2369],
                                [1622,2,0.2383],
                                [1624,2,0.2416],
                                [1626,2,0.2402],
                                [1628,2,0.2418],
                                [1630,2,0.2393],
                                [1632,2,0.2387],
                                [1634,2,0.2359],
                                [1636,2,0.2357],
                                [1638,2,0.2274],
                                [1640,2,0.2262],
                                [1642,2,0.2266],
                                [1644,2,0.2278],
                                [1646,2,0.2294],
                                [1648,2,0.2292],
                                [1650,2,0.2272],
                                [1652,2,0.2268],
                                [1654,2,0.2262],
                                [1656,2,0.226],
                                [1658,2,0.2252],
                                [1660,2,0.2245],
                                [1662,2,0.2246],
                                [1664,2,0.2227],
                                [1666,2,0.2212],
                                [1668,2,0.2193],
                                [1670,2,0.2225],
                                [1672,2,0.2173],
                                [1674,2,0.2193],
                                [1676,2,0.2161],
                                [1678,2,0.2168],
                                [1680,2,0.208],
                                [1682,2,0.2054],
                                [1684,2,0.2129],
                                [1686,2,0.2131],
                                [1688,2,0.212],
                                [1690,2,0.2105],
                                [1692,2,0.2123],
                                [1694,2,0.2112],
                                [1696,2,0.21],
                                [1698,2,0.2089],
                                [1700,2,0.2063],
                                [1702,2,0.2047],
                                [1704,2,0.2052],
                                [1706,2,0.205],
                                [1708,2,0.2017],
                                [1710,2,0.2013],
                                [1712,2,0.1982],
                                [1714,2,0.2037],
                                [1716,2,0.2022],
                                [1718,2,0.201],
                                [1720,2,0.1993],
                                [1722,2,0.1975],
                                [1724,2,0.1954],
                                [1726,2,0.1982],
                                [1728,2,0.1971],
                                [1730,2,0.1984],
                                [1732,2,0.1936],
                                [1734,2,0.1874],
                                [1736,2,0.1827],
                                [1738,2,0.1863],
                                [1740,2,0.1905],
                                [1742,2,0.1902],
                                [1744,2,0.1907],
                                [1746,2,0.1867],
                                [1748,2,0.1872],
                                [1750,2,0.1858],
                                [1752,2,0.185],
                                [1754,2,0.1856],
                                [1756,2,0.1849],
                                [1758,2,0.1843],
                                [1760,2,0.1831],
                                [1762,2,0.1793],
                                [1764,2,0.1807],
                                [1766,2,0.1817],
                                [1768,2,0.1802],
                                [1770,2,0.1791],
                                [1772,2,0.1794],
                                [1774,2,0.1792],
                                [1776,2,0.1763],
                                [1778,2,0.1747],
                                [1780,2,0.1756],
                                [1782,2,0.1747],
                                [1784,2,0.1735],
                                [1786,2,0.1739],
                                [1788,2,0.1747],
                                [1790,2,0.1733],
                                [1792,2,0.1721],
                                [1794,2,0.1709],
                                [1796,2,0.1706],
                                [1798,2,0.1703],
                                [1800,2,0.1699],
                                [1802,2,0.1672],
                                [1804,2,0.1688],
                                [1806,2,0.1688],
                                [1808,2,0.1685],
                                [1810,2,0.1686],
                                [1812,2,0.1675],
                                [1814,2,0.1658],
                                [1816,2,0.1605],
                                [1818,2,0.152],
                                [1820,2,0.1596],
                                [1822,2,0.1598],
                                [1824,2,0.1624],
                                [1826,2,0.1628],
                                [1828,2,0.1611],
                                [1830,2,0.1606],
                                [1832,2,0.1593],
                                [1834,2,0.1585],
                                [1836,2,0.1581],
                                [1838,2,0.1562],
                                [1840,2,0.1562],
                                [1842,2,0.154],
                                [1844,2,0.1541],
                                [1846,2,0.1535],
                                [1848,2,0.151],
                                [1850,2,0.1546],
                                [1852,2,0.1534],
                                [1854,2,0.1525],
                                [1856,2,0.1509],
                                [1858,2,0.1525],
                                [1860,2,0.1503],
                                [1862,2,0.1504],
                                [1864,2,0.1509],
                                [1866,2,0.1494],
                                [1868,2,0.1492],
                                [1870,2,0.1508],
                                [1872,2,0.1473],
                                [1874,2,0.1401],
                                [1876,2,0.1299],
                                [1878,2,0.1441],
                                [1880,2,0.1462],
                                [1882,2,0.1474],
                                [1884,2,0.1464],
                                [1886,2,0.1439],
                                [1888,2,0.1453],
                                [1890,2,0.1424],
                                [1892,2,0.1408],
                                [1894,2,0.1396],
                                [1896,2,0.1373],
                                [1898,2,0.139],
                                [1900,2,0.1397],
                                [1902,2,0.1409],
                                [1904,2,0.1386],
                                [1906,2,0.139],
                                [1908,2,0.1377],
                                [1910,2,0.1378],
                                [1912,2,0.1354],
                                [1914,2,0.137],
                                [1916,2,0.136],
                                [1918,2,0.1353],
                                [1920,2,0.1333],
                                [1922,2,0.135],
                                [1924,2,0.1341],
                                [1926,2,0.1344],
                                [1928,2,0.1322],
                                [1930,2,0.1313],
                                [1932,2,0.1308],
                                [1934,2,0.132],
                                [1936,2,0.1328],
                                [1938,2,0.1321],
                                [1940,2,0.1299],
                                [1942,2,0.1294],
                                [1944,2,0.1203],
                                [1946,2,0.1192],
                                [1948,2,0.1271],
                                [1950,2,0.1261],
                                [1952,2,0.1255],
                                [1954,2,0.1286],
                                [1956,2,0.1276],
                                [1958,2,0.1271],
                                [1960,2,0.1261],
                                [1962,2,0.124],
                                [1964,2,0.1222],
                                [1966,2,0.1231],
                                [1968,2,0.124],
                                [1970,2,0.1239],
                                [1972,2,0.1213],
                                [1974,2,0.1208],
                                [1976,2,0.1224],
                                [1978,2,0.1194],
                                [1980,2,0.1196],
                                [1982,2,0.1205],
                                [1984,2,0.1197],
                                [1986,2,0.1178],
                                [1988,2,0.1195],
                                [1990,2,0.1198],
                                [1992,2,0.118],
                                [1994,2,0.1162],
                                [1996,2,0.1173],
                                [1998,2,0.1159],
                                [2000,2,0.117],
                                [2002,2,0.1161],
                                [2004,2,0.1148],
                                [2006,2,0.1147],
                                [2008,2,0.1154],
                                [2010,2,0.1149],
                                [2012,2,0.1145],
                                [2014,2,0.1138],
                                [2016,2,0.1137],
                                [2018,2,0.1134],
                                [2020,2,0.1116],
                                [2022,2,0.1107],
                                [2024,2,0.1116],
                                [2026,2,0.1115],
                                [2028,2,0.1107],
                                [2030,2,0.1086],
                                [2032,2,0.1098],
                                [2034,2,0.1092],
                                [2036,2,0.1083],
                                [2038,2,0.1064],
                                [2040,2,0.1078],
                                [2042,2,0.1076],
                                [2044,2,0.1076],
                                [2046,2,0.1071],
                                [2048,2,0.1063],
                                [2050,2,0.1059],
                                [2052,2,0.1047],
                                [2054,2,0.1046],
                                [2056,2,0.1046],
                                [2058,2,0.104],
                                [2060,2,0.1028],
                                [2062,2,0.1023],
                                [2064,2,0.1005],
                                [2066,2,0.1025],
                                [2068,2,0.1019],
                                [2070,2,0.1003],
                                [2072,2,0.1004],
                                [2074,2,0.1009],
                                [2076,2,0.1006],
                                [2078,2,0.1],
                                [2080,2,0.09878],
                                [2082,2,0.09864],
                                [2084,2,0.09772],
                                [2086,2,0.09852],
                                [2088,2,0.09835],
                                [2090,2,0.09788],
                                [2092,2,0.09567],
                                [2094,2,0.09593],
                                [2096,2,0.0958],
                                [2098,2,0.0962],
                                [2100,2,0.09606],
                                [2102,2,0.09577],
                                [2104,2,0.09559],
                                [2106,2,0.09574],
                                [2108,2,0.09513],
                                [2110,2,0.09396],
                                [2112,2,0.09452],
                                [2114,2,0.09436],
                                [2116,2,0.09331],
                                [2118,2,0.09311],
                                [2120,2,0.09275],
                                [2122,2,0.09275],
                                [2124,2,0.09189],
                                [2126,2,0.09208],
                                [2128,2,0.09225],
                                [2130,2,0.09209],
                                [2132,2,0.0921],
                                [2134,2,0.09155],
                                [2136,2,0.09012],
                                [2138,2,0.0911],
                                [2140,2,0.09083],
                                [2142,2,0.09064],
                                [2144,2,0.09006],
                                [2146,2,0.08939],
                                [2148,2,0.08979],
                                [2150,2,0.08957],
                                [2152,2,0.08913],
                                [2154,2,0.08878],
                                [2156,2,0.08874],
                                [2158,2,0.08842],
                                [2160,2,0.08781],
                                [2162,2,0.08686],
                                [2164,2,0.08456],
                                [2166,2,0.07849],
                                [2168,2,0.083],
                                [2170,2,0.08557],
                                [2172,2,0.08591],
                                [2174,2,0.08592],
                                [2176,2,0.08532],
                                [2178,2,0.08425],
                                [2180,2,0.08497],
                                [2182,2,0.08425],
                                [2184,2,0.08457],
                                [2186,2,0.08465],
                                [2188,2,0.08277],
                                [2190,2,0.08304],
                                [2192,2,0.08377],
                                [2194,2,0.08349],
                                [2196,2,0.08318],
                                [2198,2,0.08299],
                                [2200,2,0.08265],
                                [2202,2,0.0823],
                                [2204,2,0.08211],
                                [2206,2,0.07966],
                                [2208,2,0.07966],
                                [2210,2,0.0808],
                                [2212,2,0.08105],
                                [2214,2,0.08072],
                                [2216,2,0.07994],
                                [2218,2,0.0797],
                                [2220,2,0.07997],
                                [2222,2,0.07962],
                                [2224,2,0.07926],
                                [2226,2,0.07811],
                                [2228,2,0.07826],
                                [2230,2,0.07831],
                                [2232,2,0.07815],
                                [2234,2,0.07802],
                                [2236,2,0.07758],
                                [2238,2,0.07648],
                                [2240,2,0.07639],
                                [2242,2,0.07642],
                                [2244,2,0.07624],
                                [2246,2,0.07612],
                                [2248,2,0.0752],
                                [2250,2,0.07541],
                                [2252,2,0.07512],
                                [2254,2,0.07402],
                                [2256,2,0.07422],
                                [2258,2,0.07441],
                                [2260,2,0.07421],
                                [2262,2,0.07299],
                                [2264,2,0.07329],
                                [2266,2,0.07315],
                                [2268,2,0.07327],
                                [2270,2,0.07297],
                                [2272,2,0.07277],
                                [2274,2,0.07252],
                                [2276,2,0.07239],
                                [2278,2,0.07242],
                                [2280,2,0.07165],
                                [2282,2,0.07007],
                                [2284,2,0.07125],
                                [2286,2,0.07124],
                                [2288,2,0.07127],
                                [2290,2,0.0711],
                                [2292,2,0.07067],
                                [2294,2,0.0692],
                                [2296,2,0.06908],
                                [2298,2,0.06919],
                                [2300,2,0.06953],
                                [2302,2,0.06955],
                                [2304,2,0.06931],
                                [2306,2,0.06923],
                                [2308,2,0.06901],
                                [2310,2,0.0687],
                                [2312,2,0.06867],
                                [2314,2,0.06826],
                                [2316,2,0.06779],
                                [2318,2,0.06745],
                                [2320,2,0.06768],
                                [2322,2,0.06675],
                                [2324,2,0.06536],
                                [2326,2,0.06559],
                                [2328,2,0.06629],
                                [2330,2,0.06616],
                                [2332,2,0.06584],
                                [2334,2,0.06571],
                                [2336,2,0.06536],
                                [2338,2,0.06496],
                                [2340,2,0.0652],
                                [2342,2,0.06539],
                                [2344,2,0.06509],
                                [2346,2,0.06486],
                                [2348,2,0.06472],
                                [2350,2,0.06453],
                                [2352,2,0.06289],
                                [2354,2,0.06239],
                                [2356,2,0.06282],
                                [2358,2,0.06266],
                                [2360,2,0.06308],
                                [2362,2,0.06305],
                                [2364,2,0.06295],
                                [2366,2,0.06284],
                                [2368,2,0.06263],
                                [2370,2,0.06211],
                                [2372,2,0.06207],
                                [2374,2,0.06066],
                                [2376,2,0.06164],
                                [2378,2,0.06192],
                                [2380,2,0.06172],
                                [2382,2,0.06098],
                                [2384,2,0.05885],
                                [2386,2,0.05908],
                                [2388,2,0.06004],
                                [2390,2,0.06029],
                                [2392,2,0.06008],
                                [2394,2,0.06003],
                                [2396,2,0.05996],
                                [2398,2,0.05989],
                                [2400,2,0.05944],
                                [2402,2,0.05965],
                                [2404,2,0.05945],
                                [2406,2,0.05919],
                                [2408,2,0.05915],
                                [2410,2,0.05902],
                                [2412,2,0.05894],
                                [2414,2,0.05734],
                                [2416,2,0.05599],
                                [2418,2,0.05748],
                                [2420,2,0.0577],
                                [2422,2,0.05767],
                                [2424,2,0.05726],
                                [2426,2,0.05717],
                                [2428,2,0.05712],
                                [2430,2,0.05712],
                                [2432,2,0.05702],
                                [2434,2,0.05641],
                                [2436,2,0.05618],
                                [2438,2,0.05599],
                                [2440,2,0.05639],
                                [2442,2,0.05617],
                                [2444,2,0.05603],
                                [2446,2,0.05498],
                                [2448,2,0.05457],
                                [2450,2,0.05462],
                                [2452,2,0.05432],
                                [2454,2,0.05455],
                                [2456,2,0.0537],
                                [2458,2,0.05392],
                                [2460,2,0.05457],
                                [2462,2,0.05442],
                                [2464,2,0.05435],
                                [2466,2,0.05405],
                                [2468,2,0.0539],
                                [2470,2,0.05285],
                                [2472,2,0.0533],
                                [2474,2,0.05313],
                                [2476,2,0.05343],
                                [2478,2,0.05303],
                                [2480,2,0.05177],
                                [2482,2,0.0514],
                                [2484,2,0.05219],
                                [2486,2,0.0516],
                                [2488,2,0.05169],
                                [2490,2,0.05225],
                                [2492,2,0.05198],
                                [2494,2,0.05175],
                                [2496,2,0.05152],
                                [2498,2,0.05154],
                                [2500,20,0.05155],
                                [2520,20,0.04984],
                                [2540,20,0.04814],
                                [2560,20,0.04672],
                                [2580,20,0.0455],
                                [2600,20,0.04457],
                                [2620,20,0.04305],
                                [2640,20,0.04211],
                                [2660,20,0.04079],
                                [2680,20,0.03968],
                                [2700,20,0.03867],
                                [2720,20,0.03763],
                                [2740,20,0.03663],
                                [2760,20,0.03546],
                                [2780,20,0.03468],
                                [2800,20,0.03385],
                                [2820,20,0.03297],
                                [2840,20,0.03209],
                                [2860,20,0.03119],
                                [2880,20,0.03032],
                                [2900,20,0.02969],
                                [2920,20,0.0289],
                                [2940,20,0.02817],
                                [2960,20,0.0275],
                                [2980,20,0.02682],
                                [3000,20,0.02612],
                                [3020,20,0.02547],
                                [3040,20,0.02465],
                                [3060,20,0.02422],
                                [3080,20,0.02364],
                                [3100,20,0.02306],
                                [3120,20,0.02246],
                                [3140,20,0.02198],
                                [3160,20,0.02144],
                                [3180,20,0.02096],
                                [3200,20,0.02048],
                                [3220,20,0.02],
                                [3240,20,0.01951],
                                [3260,20,0.01907],
                                [3280,20,0.01858],
                                [3300,20,0.01802],
                                [3320,20,0.01768],
                                [3340,20,0.01737],
                                [3360,20,0.01697],
                                [3380,20,0.01659],
                                [3400,20,0.01615],
                                [3420,20,0.01584],
                                [3440,20,0.01554],
                                [3460,20,0.0152],
                                [3480,20,0.01486],
                                [3500,20,0.01456],
                                [3520,20,0.01425],
                                [3540,20,0.01393],
                                [3560,20,0.01362],
                                [3580,20,0.01334],
                                [3600,20,0.01307],
                                [3620,20,0.01281],
                                [3640,20,0.01251],
                                [3660,20,0.01222],
                                [3680,20,0.01193],
                                [3700,20,0.01162],
                                [3720,20,0.01145],
                                [3740,20,0.01108],
                                [3760,20,0.01096],
                                [3780,20,0.01078],
                                [3800,20,0.01057],
                                [3820,20,0.01038],
                                [3840,20,0.01019],
                                [3860,20,0.00998],
                                [3880,20,0.00978],
                                [3900,20,0.0096],
                                [3920,20,0.00943],
                                [3940,20,0.00923],
                                [3960,20,0.00903],
                                [3980,20,0.00886],
                                [4000,20,0.00867],
                                [4020,20,0.00856],
                                [4040,20,0.00839],
                                [4060,20,0.00822],
                                [4080,20,0.00805],
                                [4100,20,0.00789],
                                [4120,20,0.00774],
                                [4140,20,0.00759],
                                [4160,20,0.00744],
                                [4180,20,0.00729],
                                [4200,20,0.00715],
                                [4220,20,0.00702],
                                [4240,20,0.00688],
                                [4260,20,0.00675],
                                [4280,20,0.00662],
                                [4300,20,0.0065],
                                [4320,20,0.00637],
                                [4340,20,0.00625],
                                [4360,20,0.00614],
                                [4380,20,0.00602],
                                [4400,20,0.00591],
                                [4420,20,0.0058],
                                [4440,20,0.0057],
                                [4460,20,0.00559],
                                [4480,20,0.00549],
                                [4500,20,0.00539],
                                [4520,20,0.0053],
                                [4540,20,0.0052],
                                [4560,20,0.00511],
                                [4580,20,0.00502],
                                [4600,20,0.00493],
                                [4620,20,0.00484],
                                [4640,20,0.00476],
                                [4660,20,0.00467],
                                [4680,20,0.00459],
                                [4700,20,0.00451],
                                [4720,20,0.00444],
                                [4740,20,0.00436],
                                [4760,20,0.00429],
                                [4780,20,0.00421],
                                [4800,20,0.00414],
                                [4820,20,0.00407],
                                [4840,20,0.004],
                                [4860,20,0.00394],
                                [4880,20,0.00387],
                                [4900,20,0.00381],
                                [4920,20,0.00374],
                                [4940,20,0.00368],
                                [4960,20,0.00362],
                                [4980,20,0.00356],
                                [5000,50,0.0035],
                                [5050,50,0.00339],
                                [5100,50,0.00327],
                                [5150,50,0.00315],
                                [5200,50,0.00303],
                                [5250,50,0.00292],
                                [5300,50,0.00282],
                                [5350,50,0.00272],
                                [5400,50,0.00262],
                                [5450,50,0.00253],
                                [5500,50,0.00244],
                                [5550,50,0.00236],
                                [5600,50,0.00228],
                                [5650,50,0.0022],
                                [5700,50,0.00212],
                                [5750,50,0.00205],
                                [5800,50,0.00199],
                                [5850,50,0.00192],
                                [5900,50,0.00186],
                                [5950,50,0.0018],
                                [6000,50,0.00174],
                                [6050,50,0.00169],
                                [6100,50,0.00163],
                                [6150,50,0.00158],
                                [6200,50,0.00153],
                                [6250,50,0.00149],
                                [6300,50,0.00144],
                                [6350,50,0.0014],
                                [6400,50,0.00136],
                                [6450,50,0.00132],
                                [6500,50,0.00128],
                                [6550,50,0.00124],
                                [6600,50,0.0012],
                                [6650,50,0.00117],
                                [6700,50,0.00114],
                                [6750,50,0.0011],
                                [6800,50,0.00107],
                                [6850,50,0.00104],
                                [6900,50,0.00101],
                                [6950,50,0.00099],
                                [7000,50,0.00096],
                                [7050,50,0.00093],
                                [7100,50,0.00091],
                                [7150,50,0.00088],
                                [7200,50,0.00086],
                                [7250,50,0.00084],
                                [7300,50,0.00082],
                                [7350,50,0.00079],
                                [7400,50,0.00077],
                                [7450,50,0.00075],
                                [7500,50,0.00073],
                                [7550,50,0.00072],
                                [7600,50,0.0007],
                                [7650,50,0.00068],
                                [7700,50,0.00066],
                                [7750,50,0.00065],
                                [7800,50,0.00063],
                                [7850,50,0.00062],
                                [7900,50,0.0006],
                                [7950,50,0.00059],
                                [8000,50,0.00057],
                                [8050,50,0.00056],
                                [8100,50,0.00055],
                                [8150,50,0.00053],
                                [8200,50,0.00052],
                                [8250,50,0.00051],
                                [8300,50,0.0005],
                                [8350,50,0.00048],
                                [8400,50,0.00047],
                                [8450,50,0.00046],
                                [8500,50,0.00045],
                                [8550,50,0.00044],
                                [8600,50,0.00043],
                                [8650,50,0.00042],
                                [8700,50,0.00041],
                                [8750,50,0.0004],
                                [8800,50,0.0004],
                                [8850,50,0.00039],
                                [8900,50,0.00038],
                                [8950,50,0.00037],
                                [9000,50,0.00036],
                                [9050,50,0.00036],
                                [9100,50,0.00035],
                                [9150,50,0.00034],
                                [9200,50,0.00033],
                                [9250,50,0.00033],
                                [9300,50,0.00032],
                                [9350,50,0.00031],
                                [9400,50,0.00031],
                                [9450,50,0.0003],
                                [9500,50,0.00029],
                                [9550,50,0.00029],
                                [9600,50,0.00028],
                                [9650,50,0.00028],
                                [9700,50,0.00027],
                                [9750,50,0.00027],
                                [9800,50,0.00026],
                                [9850,50,0.00026],
                                [9900,50,0.00025],
                                [9950,50,0.00025],
                                [10000,1000,0.00024],
                                [11000,1000,0.00016],
                                [12000,1000,0.00012],
                                [13000,1000,0.00008],
                                [14000,1000,0.00006],
                                [15000,1000,0.00005],
                                [16000,1000,0.00004],
                                [17000,1000,0.00003],
                                [18000,1000,0.00002],
                                [19000,1000,0.00002],
                                [20000,5000,0.00001],
                                [25000,5000,0.00001],
                                [30000,5000,0],
                                [35000,5000,0],
                                [40000,10000,0],
                                [50000,10000,0],
                                [60000,20000,0],
                                [80000,20000,0],
                                [100000,20000,0],
                                [120000,30000,0],
                                [150000,50000,0],
                                [200000,50000,0],
                                [250000,50000,0],
                                [300000,100000,0],
                                [400000,600000,0],
                                [1000000,0,0]])
        return spectrum

    def calcSolarDistance(self, julianDay):
        if ((julianDay < 1) or (julianDay > 366)):
            raise ARCSIException("Julian Day must be between 1 and 366.")
        radiansJulianDay = ((julianDay - 4)*0.9856)*(math.pi/180);
        solarDist = 1 - 0.01672 * math.cos(radiansJulianDay);
        return solarDist;

    def calcSolarIrradiance(self, solarSpec, respFuncs, julianDay):
        minWv = respFuncs[0][0]
        maxWv = respFuncs[len(respFuncs)-1][0]

        rangeWv = int(maxWv - minWv)+1

        solarWVs = solarSpec[:,[0]]
        solarSpecSub = numpy.where(((solarWVs >= (minWv-100)) & (solarWVs <= (maxWv+100))), solarSpec, numpy.nan)
        solarSpecSub = solarSpecSub[~numpy.isnan(solarSpecSub).any(axis=1)]

        partA = 0
        partB = 0

        for i in range(rangeWv):
            wv = minWv + float(i)
            rfDist = 0
            minDist = 0
            minDistRF = None
            first = True
            for j in range(len(respFuncs)):
                rfDist = math.fabs(wv - respFuncs[j][0])
                if first:
                    minDist = rfDist
                    minDistRF = respFuncs[j]
                    first = False
                else:
                    if rfDist < minDist:
                        minDist = rfDist
                        minDistRF = respFuncs[j]
            minDist = 0
            minDistSS = None
            first = True
            for j in range(len(solarSpecSub)):
                ssDist = math.fabs(wv - solarSpecSub[j][0])
                if first:
                    minDist = ssDist
                    minDistSS = solarSpecSub[j]
                    first = False
                else:
                    if ssDist < minDist:
                        minDist = ssDist
                        minDistSS = solarSpecSub[j]

            partA = partA + (minDistSS[2]*minDistRF[1])
            partB = partB + minDistRF[1]

        solarDist = self.calcSolarDistance(julianDay)

        irradiance = ((partA * solarDist)/(partB * solarDist))*1000

        return irradiance

    def run(self, inputFile, seperator, ignoreLines, wvCol, respCol, julianDay):
        solarSpec = self.getE490SolarSpectrum()
        arcsiUtils = ARCSIUtils()
        respFunc = arcsiUtils.readSpectralResponseFunc(inputFile, seperator, ignoreLines, wvCol, respCol)
        irradiance = self.calcSolarIrradiance(solarSpec, respFunc, julianDay)
        print ("Irradiance: " + str(round(irradiance,2)))

if __name__ == '__main__':
    """
    The command line user interface to ARCSI Irradiance tool.
    """
    parser = argparse.ArgumentParser(prog='arcsiirradiance.py',
                                    description='''ARCSI command for calculating
                                                the solar exoatmospheric
                                                irradiance (ESUN) for a spectral
                                                response function.''',
                                    epilog='''Using the E-490 Solar Spectrum and a
                                              nearest neighbour interpolation to
                                              calculate the exoatmospheric
                                              irradiance (ESUN) value for a specific
                                              spectral response function. The ESUN value
                                              is require to convert at sensor radiance
                                              to at sensor reflectance or top of
                                              atmosphere reflectance (TOA)''')
    # Request the version number.
    parser.add_argument('-v', '--version', action='version', version='%(prog)s version ' + ARCSI_VERSION)
    # Define the argument for specifying the input spectral response file.
    parser.add_argument("-i", "--input", type=str, required=True,
                        help='''A seperated (--sep) text file defining the
                        wavelength (nm) and normalised spectral response
                        function.''')
    # Define the argument for specifying input seperator.
    parser.add_argument("-s", "--sep", type=str,
                        help='''The seperator used to split the columns within the input
                        file. The default is space seperated where consecutive spaces
                        are ignored.''')
    # Define the argument for specifying the number of lines to ignore.
    parser.add_argument("--ignore", type=int, default="0",
                        help='''Number of lines at the beginning of the file
                        which should be ignored.''')
    # Define the argument for specifying the column for the wavelength.
    parser.add_argument("--wvcol", type=int, default="0",
                        help='''The wavelength column within the input file (starting at 0).''')
    # Define the argument for specifying the column for the response function.
    parser.add_argument("--rcol", type=int, default="1",
                        help='''The response function column within the input file (starting at 0).''')
    # Define the argument for specifying the julian day.
    parser.add_argument("-j","--julianday", type=int, default="100",
                        help='''The julian day of the acquasition.''')
    # Call the parser to parse the arguments.
    args = parser.parse_args()

    arcsiObj = ARCSISolarIrradiance()
    arcsiObj.run(args.input, args.sep, args.ignore, args.wvcol, args.rcol, args.julianday)











